@@ -2,7 +2,7 @@ package Font::TTF::Ttopen;
 
 =head1 NAME
 
-Font::TTF::Ttopen - Opentype superclass for standard Opentype lookup based tables
+Font::TTF::ttopen - Opentype superclass for standard Opentype lookup based tables
 (GSUB and GPOS)
 
 =head1 DESCRIPTION
@@ -315,7 +315,7 @@ sub read
     	{
     	    if ($tag =~ m/(.*?)\s_(\d+)$/o)
     	    { $tag = $1 . " _" . ($2 + 1); }
-    	    else
+    	    elsef
     	    { $tag .= " _0"; }
     	}
 	    $l->{$tag}{' OFFSET'} = $off + $oFeat;
@@ -385,7 +385,7 @@ sub read
        	    next unless $l->{$tag}{$lTag}{' REFTAG'};
     	    $temp = $l->{$tag}{$lTag}{' REFTAG'};
     	    $l->{$tag}{$lTag} = &copy($l->{$tag}{$temp});
-    	    $l->{$tag}{$lTag}{' REFTAG'} = $temp;
+    	    $l->{$tag}{' REFTAG'} = $temp;
     	}
     }
     foreach $tag (keys %$l)
@@ -466,10 +466,12 @@ sub out
 
 # First sort the features
     $i = 0;
-    $self->{'FEATURES'}{'FEAT_TAGS'} = [sort grep {length($_) == 4 || m/\s_\d+$/o} %{$self->{'FEATURES'}}]
-            if (!defined $self->{'FEATURES'}{'FEAT_TAGS'});
-    foreach $t (@{$self->{'FEATURES'}{'FEAT_TAGS'}})
-    { $self->{'FEATURES'}{$t}{'INDEX'} = $i++; }
+    foreach $t (sort grep {length($_) == 4 || m/\s_\d+$/o} %{$self->{'FEATURES'}})
+    {
+        $self->{'FEATURES'}{$t}{'INDEX'} = $i++;
+        push (@tags, $t);
+    }
+    $self->{'FEATURES'}{'FEAT_TAGS'} = \@tags;
 
     $base = $fh->tell();
     $fh->print(TTF_Pack("f", $self->{'Version'}));
@@ -492,16 +494,11 @@ sub out
     	{ $fh->print(pack("a4n", $lTag, 0)); }
     	foreach $lTag (@{$tag->{'LANG_TAGS'}}, 'DEFAULT')
     	{
-    	    my ($def);
     	    $l = $tag->{$lTag};
     	    next if (!defined $l || $l->{' REFTAG'} ne '');
     	    $l->{' OFFSET'} = tell($fh) - $base - $oScript - $tag->{' OFFSET'};
-    	    if (defined $l->{'DEFAULT'})
-#    	    { $def = $self->{'FEATURES'}{$l->{'FEATURES'}[$l->{'DEFAULT'}]}{'INDEX'}; }
-            { $def = $l->{'DEFAULT'}; }
-    	    else
-    	    { $def = -1; }
-    	    $fh->print(pack("n*", $l->{'RE_ORDER'}, $def, $#{$l->{'FEATURES'}} + 1,
+    	    $fh->print(pack("n*", $l->{'RE_ORDER'}, defined $l->{'DEFAULT'} ? $l->{'DEFAULT'} : -1,
+    	            $#{$l->{'FEATURES'}} + 1,
     	            map {$self->{'FEATURES'}{$_}{'INDEX'}} @{$l->{'FEATURES'}}));
     	}
     	$end = $fh->tell();
@@ -549,49 +546,39 @@ sub out
     undef $big;
     $fh->seek($end, 0);
     $oLook = $end - $base;
-    
-    # Start Lookup List Table
     $nTags = $#{$self->{'LOOKUP'}} + 1;
     $fh->print(pack("n", $nTags));
     $fh->print(pack("n", 0) x $nTags);
-    $end = $fh->tell();		# end of LookupListTable = start of Lookups
+    $end = $fh->tell();
     foreach $tag (@{$self->{'LOOKUP'}})
     { $nSubs += $self->num_sub($tag); }
     for ($i = 0; $i < $nTags; $i++)
     {
         $fh->seek($end, 0);
         $tag = $self->{'LOOKUP'}[$i];
-        $off = $end - $base - $oLook;	# BH 2004-03-04
-        # Is there room, from the start of this i'th lookup, for this and the remaining
-        # lookups to be wrapped in extension lookups?
-        if (!defined $big && $off + ($nTags - $i) * 6 + $nSubs * 10 > 65535) # BH 2004-03-04
+        $tag->{' OFFSET'} = $end - $base - $oLook;
+        if (!defined $big && $tag->{' OFFSET'} + ($nTags - $i) * 6 + $nSubs * 10 > 65535)
         {
-			# Not enough room -- need to start an extension!            
             my ($k, $ext);
             $ext = $self->extension();
-            # Must turn previous lookup into the first extension
             $i--;
             $tag = $self->{'LOOKUP'}[$i];
             $end = $tag->{' OFFSET'} + $base + $oLook;
             $fh->seek($end, 0);
             $big = $i;
-            # For this and the remaining lookups, build extensions lookups
             for ($j = $i; $j < $nTags; $j++)
             {
                 $tag = $self->{'LOOKUP'}[$j];
                 $nSub = $self->num_sub($tag);
                 $fh->print(pack("nnn", $ext, $tag->{'FLAG'}, $nSub));
-                $fh->print(pack("n*", map {$_ * 8 + 6 + $nSub * 2} (0 .. $nSub-1)));	# BH 2004-03-04
-                $tag->{' EXT_OFFSET'} = $fh->tell();	# = first extension lookup subtable
-                $tag->{' OFFSET'} = $tag->{' EXT_OFFSET'} - $nSub * 2 - 6 - $base - $oLook; # offset to this extension lookup
+                $fh->print(pack("n*", map {$_ * 8 + 6 + $nSub * 2} (1 .. $nSub)));
+                $tag->{' EXT_OFFSET'} = $fh->tell();
+                $tag->{' OFFSET'} = $tag->{' EXT_OFFSET'} - $nSub * 2 - 6 - $base - $oLook;
                 for ($k = 0; $k < $nSub; $k++)
                 { $fh->print(pack('nnN', 1, $tag->{'TYPE'}, 0)); }
             }
-            
             $tag = $self->{'LOOKUP'}[$i];
-            # Leave file positioned after all the extension lookups -- where the referenced lookups will start.
         }
-        $tag->{' OFFSET'} = $off unless defined $big;	# BH 2004-03-04
         $nSub = $self->num_sub($tag);
         if (!defined $big)
         {
@@ -1039,16 +1026,16 @@ sub out_context
     } elsif ($type == 6 && $fmt == 3)
     {
         $r = $lookup->{'RULES'}[0][0];
-        $out .= pack('n2', $fmt, scalar @{$r->{'PRE'}});
+        $out .= pack('n2', $fmt, $#{$r->{'PRE'}} + 1);
         foreach $t (@{$r->{'PRE'}})
         { $out .= Font::TTF::Ttopen::ref_cache($t, $ctables, length($out)); }
-        $out .= pack('n', scalar @{$r->{'MATCH'}});
+        $out .= pack('n', $#{$r->{'MATCH'}} + 1);
         foreach $t (@{$r->{'MATCH'}})
         { $out .= Font::TTF::Ttopen::ref_cache($t, $ctables, length($out)); }
-        $out .= pack('n', scalar @{$r->{'POST'}});
+        $out .= pack('n', $#{$r->{'POST'}} + 1);
         foreach $t (@{$r->{'POST'}})
         { $out .= Font::TTF::Ttopen::ref_cache($t, $ctables, length($out)); }
-        $out .= pack('n', scalar @{$r->{'ACTION'}});
+        $out .= pack('n', $#{$r->{'ACTION'}} + 1);
         foreach $t (@{$r->{'ACTION'}})
         { $out .= pack('n2', @$t); }
     }
